:fullscreen, ::backdrop {
  background-color: #F8FAFB;
  padding: 0 1rem;
  overflow: scroll !important;
}

.dashboard-container {
  margin-bottom: 1rem;
}

.dashboard-header {
  display: flex;
  flex-direction: row;
  flex-wrap: wrap;
  justify-content: space-between;
  align-items: center;
  margin-bottom: 1rem;
}

.dashboard-toolbar {
  margin-bottom: 1rem;
}

.dashboard-filters {
  margin-bottom: 1rem;
}

.dashboard-filters > form input[type="text"] {
  all: revert;
}

.dashboard-filters-boxes {
  display: flex;
  flex-direction: row;
  flex-wrap: wrap;
}

.dashboard-filters-boxes > fieldset {
  margin: 0rem 1rem 0.5rem 0rem;
  padding: 0rem 1rem 0.5rem 1rem;
  border: 1px solid #a0a0a0;
  border-radius: 4px;
}

.dashboard-filters-boxes > fieldset > legend {
  padding: 0 0.2rem 0 0.2rem;
  font-size: small;
}

.dashboard-grid {
  display: grid;
  grid-auto-columns: minmax(100px, 1fr);
  grid-gap: 0.5rem;
}

.dashboard-card {
  background-color: white;
  padding: 1rem;
  box-shadow: 0px 1px 3px rgba(0, 0, 0, .13);
  border-color: transparent;
  border-radius: 8px;
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  align-items: flex-start;
}

.dashboard-card-title {
  font-size: small;
  font-weight: bold;
}

.dashboard-card-title a {
  text-decoration: none;
}

.dashboard-card-chart {
  width: 100%;
  height: 100%;
}

.dashboard-card-chart ul {
  list-style: initial;
  list-style-position: inside;
}

.dashboard-card-chart ol {
  list-style: initial;
  list-style-position: inside;
  list-style-type: decimal;
}

.chart-container {
  width: 100%;
  height: 70vh;
}

#vg-tooltip-element * {
  all: revert;
}

.chart-tooltip {
  visibility: hidden;
  position: relative;
  display: inline-block;
}
